@@ -510,7 +510,9 @@ class Collection(object):
                     cache.delete_document(entry.idterm)
                     self.remove_file(entry.fname)
                 else:
-                    logger.warning("No such entry: {}, ignoring".format(eid))
+                    logger.warning("Not removing {}: no such entry".format(
+                        eid
+                    ))
         cache.commit()
         cache.close()
 
@@ -561,7 +563,7 @@ class Collection(object):
         except jinja2.exceptions.TemplateSyntaxError as e:
             raise TemplatingError(e)
         try:
-            data = ruamel.yaml.load(rendered, ruamel.yaml.RoundTripLoader)
+            data = self.yaml.load(rendered)
         except ruamel.yaml.YAMLError as e:
             logger.warning(
                 "The following data failed to load as YAML: \n{}".format(
