@@ -4,61 +4,14 @@ import os
 import subprocess
 import sys
 
-import ruamel.yaml
+try:
+    import argcomplete
+except ImportError:
+    argcomplete = False
 
 from . import Collection, Entry, TemplatingError
 
-
-def edit_file_in_external_editor(filepath):
-    # First we try to use $EDITOR
-    editor = os.environ.get('EDITOR')
-    if editor:
-        try:
-            subprocess.call([editor, filepath])
-        except FileNotFoundError as e:
-            if editor in str(e):
-                logging.info(
-                    'Could not open file {} with $EDITOR (currently {})'
-                    .format(
-                        filepath, editor
-                    )
-                )
-            else:
-                logging.warning("Could not open file {}".format(filepath))
-                return False
-        else:
-            return True
-    # then we try to use sensible-editor (which should be available on
-    # debian and derivatives)
-    try:
-        subprocess.call(['sensible-editor', filepath])
-    except FileNotFoundError as e:
-        if 'sensible-editor' in e.strerror:
-            logging.debug(
-                "Could not open file {} with editor: sensible-editor".format(
-                    filepath
-                )
-            )
-        else:
-            logging.warning("Could not open file {}".format(filepath))
-            return False
-    else:
-        return True
-    # and finally we fallback to vi, because ed is the standard editor,
-    # but that would be way too cruel, and vi is also in posix
-    try:
-        subprocess.call(['vi', filepath])
-    except FileNotFoundError as e:
-        if 'vi' in e.strerror:
-            logging.warning(
-                "Could not open file {} with any known editor".format(filepath)
-            )
-            return False
-        else:
-            logging.warning("Could not open file {}".format(filepath))
-            return False
-    else:
-        return True
+logger = logging.getLogger(__name__)
 
 
 def _get_first_docstring_line(obj):
@@ -77,15 +30,28 @@ class MainCommand:
     def main(self):
         desc = _get_first_docstring_line(self)
         self.parser = argparse.ArgumentParser(description=desc)
+        self.parser.add_argument('--verbose', '-v',
+                                 action='store_true',
+                                 help="Display debug messages")
         self.parser.set_defaults(func=self._main)
         self.subparsers = self.parser.add_subparsers()
         for name, sub in self.commands:
             sub_help = _get_first_docstring_line(sub)
-            s_parser = self.subparsers.add_parser(name, help=sub_help)
+            s_parser = self.subparsers.add_parser(
+                name,
+                help=sub_help,
+                description=sub.__doc__,
+            )
             for arg in sub.arguments:
                 s_parser.add_argument(*arg[0], **arg[1])
             s_parser.set_defaults(func=sub._main)
+        if argcomplete:
+            argcomplete.autocomplete(self.parser)
         self.args = self.parser.parse_args()
+
+        if self.args.verbose:
+            logging.getLogger('lesana').setLevel(logging.DEBUG)
+
         self.args.func(self.args)
 
 
@@ -98,6 +64,58 @@ class Command:
         self.args = args
         self.main()
 
+    def edit_file_in_external_editor(self, filepath):
+        # First we try to use $EDITOR
+        editor = os.environ.get('EDITOR')
+        if editor:
+            try:
+                subprocess.call([editor, filepath])
+            except FileNotFoundError as e:
+                if editor in str(e):
+                    logger.info(
+                        'Could not open file {} with $EDITOR (currently {})'
+                        .format(
+                            filepath, editor
+                        )
+                    )
+                else:
+                    logger.warning("Could not open file {}".format(filepath))
+                    return False
+            else:
+                return True
+        # then we try to use sensible-editor (which should be available on
+        # debian and derivatives)
+        try:
+            subprocess.call(['sensible-editor', filepath])
+        except FileNotFoundError as e:
+            if 'sensible-editor' in e.strerror:
+                logger.debug(
+                    "Could not open file {} with editor: sensible-editor"
+                    .format(filepath)
+                )
+            else:
+                logger.warning("Could not open file {}".format(filepath))
+                return False
+        else:
+            return True
+        # and finally we fallback to vi, because ed is the standard editor,
+        # but that would be way too cruel, and vi is also in posix
+        try:
+            subprocess.call(['vi', filepath])
+        except FileNotFoundError as e:
+            if 'vi' in e.strerror:
+                logger.warning(
+                    "Could not open file {} with any known editor".format(
+                        filepath
+                    )
+                )
+                return False
+            else:
+                logger.warning("Could not open file {}".format(filepath))
+                return False
+        else:
+            return True
+
 
 class New(Command):
     """
@@ -123,7 +141,7 @@ class New(Command):
         new_entry = self.entry_class(collection)
         collection.save_entries([new_entry])
         filepath = os.path.join(collection.itemdir, new_entry.fname)
-        if edit_file_in_external_editor(filepath):
+        if self.edit_file_in_external_editor(filepath):
             collection.update_cache([filepath])
             if self.args.git:
                 collection.git_add_files([filepath])
@@ -166,7 +184,7 @@ class Edit(Command):
         collection.save_entries([entry])
         # and then edit the updated file
         filepath = os.path.join(collection.itemdir, entry.fname)
-        if edit_file_in_external_editor(filepath):
+        if self.edit_file_in_external_editor(filepath):
             collection.update_cache([filepath])
             if self.args.git:
                 collection.git_add_files([filepath])
@@ -204,7 +222,7 @@ class Show(Command):
             try:
                 print(entry.render(self.args.template))
             except TemplatingError as e:
-                logging.error("{}".format(e))
+                logger.error("{}".format(e))
                 sys.exit(1)
         else:
             print(entry.yaml_data)
@@ -272,37 +290,52 @@ class Search(Command):
             ['--sort'],
             dict(action='append', help='Sort results by a sortable field'),
         ),
+        (
+            ['--expand-query-template', '-e'],
+            {
+                'action': 'store_true',
+                'help':
+                    'Render search_aliases in the query as a jinja2 template',
+            },
+        ),
         (
             ['query'],
-            dict(help='Xapian query to search in the collection', nargs='+'),
+            {
+                'help': 'Xapian query to search in the collection',
+                'nargs': '*',
+                'default': '*',
+            },
         ),
     ]
 
     def main(self):
         # TODO: implement "searching" for everything
         if self.args.offset:
-            logging.warning(
+            logger.warning(
                 "offset exposes an internal knob and MAY BE REMOVED "
                 + "from a future release of lesana"  # noqa: W503
             )
         if self.args.pagesize:
-            logging.warning(
+            logger.warning(
                 "pagesize exposes an internal knob and MAY BE REMOVED "
                 + "from a future release of lesana"  # noqa: W503
             )
         offset = self.args.offset or 0
         pagesize = self.args.pagesize or 12
         collection = self.collection_class(self.args.collection)
+        query = self.args.query
+        if self.args.expand_query_template:
+            query = collection.render_query_template(query)
         # sorted results require a less efficient full search rather
         # than being able to use the list of all documents.
-        if self.args.query == ['*'] and not (
+        if query == ['*'] and not (
             self.args.sort
             or getattr(collection.settings, 'default_sort', False)
         ):
             results = collection.get_all_documents()
         else:
             collection.start_search(
-                ' '.join(self.args.query),
+                ' '.join(query),
                 sort_by=self.args.sort
             )
             if self.args.all:
@@ -314,13 +347,69 @@ class Search(Command):
                 template = collection.get_template(self.args.template)
                 print(template.render(entries=results))
             except TemplatingError as e:
-                logging.error("{}".format(e))
+                logger.error("{}".format(e))
                 sys.exit(1)
         else:
             for entry in results:
                 print("{entry}".format(entry=entry,))
 
 
+class GetValues(Command):
+    """
+    List all values for one field, with entry counts.
+    """
+    arguments = [
+        (
+            ['--collection', '-c'],
+            {
+                'help': 'The collection to work on (default .)'
+            },
+        ),
+        (
+            ['--field', '-f'],
+            {
+                'help': 'Name of the field',
+                'required': True,
+            },
+        ),
+        (
+            ['--template', '-t'],
+            {
+                'help': 'Template to use when displaying results',
+            },
+        ),
+        (
+            ['query'],
+            {
+                'help': 'Xapian query to limit the count search " \
+                      + "in the collection',
+                'nargs': '*',
+                'default': '*'
+            },
+        ),
+    ]
+
+    def main(self):
+        collection = self.collection_class(self.args.collection)
+        counts = collection.get_field_values(
+            self.args.field,
+            ' '.join(self.args.query)
+        )
+        if self.args.template:
+            try:
+                template = collection.get_template(self.args.template)
+                print(template.render(counts=counts))
+            except TemplatingError as e:
+                logger.error("{}".format(e))
+                sys.exit(1)
+        else:
+            for v in counts:
+                print("{value}: {count}".format(
+                    value=v['value'],
+                    count=v['frequency']
+                ))
+
+
 class Export(Command):
     """
     Export entries to a different collection
@@ -347,21 +436,19 @@ class Export(Command):
             collection.start_search(' '.join(self.args.query))
             results = collection.get_all_search_results()
         for entry in results:
+            data = {
+                "entry": entry
+            }
+            data.update(entry.data)
             try:
-                template = collection.get_template(self.args.template)
-                rendered = template.render(entry=entry, **entry.data)
+                destination.entry_from_rendered_template(
+                    self.args.template,
+                    data
+                )
             except TemplatingError as e:
-                logging.error("Error converting entry: {}".format(entry))
-                logging.error("{}".format(e))
+                logger.error("Error converting entry: {}".format(entry))
+                logger.error("{}".format(e))
                 sys.exit(1)
-            try:
-                data = ruamel.yaml.load(rendered, ruamel.yaml.RoundTripLoader)
-            except ruamel.yaml.YAMLError as e:
-                logging.error("Error converting entry: {}".format(entry))
-                logging.error("{}".format(e))
-                sys.exit(1)
-            e = self.entry_class(destination, data=data)
-            destination.save_entries([e])
 
 
 class Init(Command):
@@ -387,7 +474,7 @@ class Init(Command):
         self.collection_class.init(
             self.args.collection,
             git_enabled=self.args.git,
-            edit_file=edit_file_in_external_editor,
+            edit_file=self.edit_file_in_external_editor,
         )
 
 
