@@ -1,4 +1,5 @@
 import datetime
+import decimal
 import logging
 import os.path
 import shutil
@@ -37,15 +38,20 @@ class testEntries(unittest.TestCase):
         self.assertEqual(entry.idterm, 'Q' + eid)
         self.assertEqual(entry.short_id, eid[:8])
 
+    @unittest.skipIf(
+        ruamel.yaml.version_info < (0, 16, 0),
+        "Preservation of data on file requires ruamel.yaml >= 0.16",
+    )
     def test_write_new(self):
         new_entry = lesana.Entry(self.collection)
         self.collection.save_entries(entries=[new_entry])
         entry_fname = os.path.join(self.basepath, new_entry.fname)
         with open(entry_fname) as fp:
             text = fp.read()
-        self.assertIn('quantity (integer): how many items are there', text)
-        self.assertIn('other (yaml):', text)
-        self.assertNotIn('position (string)', text)
+        self.assertIn('# quantity (integer): how many items are there', text)
+        self.assertIn('# other (yaml):', text)
+        self.assertNotIn('# position (string)', text)
+        self.assertNotIn('# # other (yaml)', text)
         written = ruamel.yaml.safe_load(text)
         self.assertIsInstance(written['quantity'], int)
         self.assertIsInstance(written['name'], str)
@@ -281,6 +287,85 @@ class testSimpleCollection(unittest.TestCase):
         with open(fname, 'r') as fp:
             self.assertEqual(entry.yaml_data, fp.read())
 
+    def test_list_values(self):
+        values = self.collection.get_field_values('position')
+        values = list(values)
+        self.assertEqual(len(values), 2)
+        self.assertEqual(values, [
+            {'value': 'somewhere', 'frequency': 2},
+            {'value': None, 'frequency': 1},
+        ])
+
+    def test_entry_from_template(self):
+        # TODO: make finding the templates less prone to breaking and
+        # then remove the cwd change from here
+        old_cwd = os.getcwd()
+        os.chdir(self.tmpdir)
+        data = {
+            "name": "This is a name",
+        }
+        entry = self.collection.entry_from_rendered_template(
+            "templates/new_entry_from_data.yaml",
+            data
+        )
+        os.chdir(old_cwd)
+        self.assertIsInstance(entry, lesana.Entry)
+        self.assertEqual(entry.data["name"], "This is a name")
+
+    def test_entry_from_template_multiple_data_sources(self):
+        # TODO: make finding the templates less prone to breaking and
+        # then remove the cwd change from here
+        old_cwd = os.getcwd()
+        os.chdir(self.tmpdir)
+        data = {
+            "name": "This is a name",
+        }
+        values = {
+            "quantity": 5,
+            "cost": decimal.Decimal("3.5"),
+        }
+        entry = self.collection.entry_from_rendered_template(
+            "templates/new_entry_from_multiple_data.yaml",
+            {
+                "data": data,
+                "values": values
+            }
+        )
+        os.chdir(old_cwd)
+        self.assertIsInstance(entry, lesana.Entry)
+        self.assertEqual(entry.data["name"], "This is a name")
+        self.assertEqual(entry.data["quantity"], 5)
+
+    def test_entry_from_bad_template(self):
+        # TODO: make finding the templates less prone to breaking and
+        # then remove the cwd change from here
+        old_cwd = os.getcwd()
+        os.chdir(self.tmpdir)
+        data = {
+            "name": "This is a name",
+        }
+        with self.assertRaises(lesana.collection.TemplatingError):
+            self.collection.entry_from_rendered_template(
+                "templates/new_entry_from_data_broken.yaml",
+                data
+            )
+        os.chdir(old_cwd)
+
+    def test_entry_from_bad_yaml(self):
+        # TODO: make finding the templates less prone to breaking and
+        # then remove the cwd change from here
+        old_cwd = os.getcwd()
+        os.chdir(self.tmpdir)
+        data = {
+            "name": "This is a name",
+        }
+        with self.assertRaises(lesana.collection.TemplatingError):
+            self.collection.entry_from_rendered_template(
+                "templates/new_entry_from_data_invalid_yaml.yaml",
+                data
+            )
+        os.chdir(old_cwd)
+
 
 class testComplexCollection(unittest.TestCase):
     def setUp(self):
@@ -297,13 +382,13 @@ class testComplexCollection(unittest.TestCase):
             self.collection.settings['name'],
             "Fully featured lesana collection",
         )
-        self.assertEqual(len(self.collection.settings['fields']), 14)
+        self.assertEqual(len(self.collection.settings['fields']), 16)
         self.assertIsNotNone(self.collection.stemmer)
-        self.assertEqual(len(self.collection.indexed_fields), 8)
+        self.assertEqual(len(self.collection.indexed_fields), 9)
 
     def test_index(self):
         indexed = self.collection.update_cache()
-        self.assertEqual(indexed, 9)
+        self.assertEqual(indexed, 10)
 
     def test_indexing_list(self):
         self.collection.update_cache(['73097121f1874a6ea2f927db7dc4f11e.yaml'])
@@ -351,6 +436,10 @@ class testComplexCollection(unittest.TestCase):
         for f in to_test:
             self.assertIsInstance(self.collection.fields[f[0]], f[1])
 
+    @unittest.skipIf(
+        ruamel.yaml.version_info < (0, 16, 0),
+        "Preservation of data on file requires ruamel.yaml >= 0.16",
+    )
     def test_comments_are_preserved(self):
         e = self.collection.entry_from_eid('5084bc6e94f24dc6976629282ef30419')
         yaml_data = e.yaml_data
@@ -361,6 +450,10 @@ class testComplexCollection(unittest.TestCase):
             yaml_data.endswith("# and a comment at the end\n")
         )
 
+    @unittest.skipIf(
+        ruamel.yaml.version_info < (0, 16, 0),
+        "Preservation of data on file requires ruamel.yaml >= 0.16",
+    )
     def test_data_is_stored_as_written_on_file(self):
         e = self.collection.entry_from_eid('5084bc6e94f24dc6976629282ef30419')
         fname = 'tests/data/complex/items/' + \
@@ -412,13 +505,13 @@ class testComplexCollection(unittest.TestCase):
         self.collection.start_search('*')
         res = self.collection.get_search_results()
         matches = list(res)
-        self.assertEqual(len(matches), 9)
+        self.assertEqual(len(matches), 10)
         for i in range(5):
             self.assertEqual(matches[i].data['order'], None)
-        self.assertEqual(matches[5].data['order'], 'alpha')
-        self.assertEqual(matches[6].data['order'], 'charlie')
-        self.assertEqual(matches[7].data['order'], 'delta')
-        self.assertEqual(matches[8].data['order'], 'zucchini')
+        self.assertEqual(matches[6].data['order'], 'alpha')
+        self.assertEqual(matches[7].data['order'], 'charlie')
+        self.assertEqual(matches[8].data['order'], 'delta')
+        self.assertEqual(matches[9].data['order'], 'zucchini')
 
     def test_update_entry(self):
         eid = '5084bc6e94f24dc6976629282ef30419'
@@ -443,6 +536,33 @@ class testComplexCollection(unittest.TestCase):
         self.assertEqual(entry.data['updated'].year, now.year)
         self.assertEqual(entry.data['version'], 2)
 
+    def test_list_values(self):
+        values = self.collection.get_field_values('position')
+        values = list(values)
+        self.assertEqual(values, [
+            {'value': b'Somewhere', 'frequency': 1},
+            {'value': b'over there', 'frequency': 1},
+            {'value': b'somewhere', 'frequency': 1},
+            {'value': b'there', 'frequency': 1},
+        ])
+
+    def test_decimal_as_float(self):
+        eid = 'a4265cc5dfa94c3d8030d7df4a0ab747'
+        entry = self.collection.entry_from_eid(eid)
+        self.assertEqual(entry.data['price'], "1.90")
+
+    def test_search_aliases(self):
+        search_query = "{{ nice }}"
+        search_query = self.collection.render_query_template(search_query)
+        print("QUERY IS", search_query)
+        self.collection.start_search(search_query)
+        res = self.collection.get_search_results()
+        matches = list(res)
+        self.assertEqual(len(matches), 2)
+        matches_ids = [m.eid for m in matches]
+        self.assertIn('8e9fa1ed3c1b4a30a6be7a98eda0cfa7', matches_ids)
+        self.assertIn('5084bc6e94f24dc6976629282ef30419', matches_ids)
+
 
 class testCollectionWithErrors(unittest.TestCase):
     def setUp(self):
@@ -472,7 +592,7 @@ class testCollectionWithErrors(unittest.TestCase):
         self.assertIsNotNone(self.collection.settings)
         self.assertIsNotNone(self.collection.stemmer)
         # Fields with no "index" entry are not indexed
-        self.assertEqual(len(self.collection.settings['fields']), 8)
+        self.assertEqual(len(self.collection.settings['fields']), 10)
         self.assertEqual(len(self.collection.indexed_fields), 3)
 
     def test_init(self):
@@ -481,7 +601,7 @@ class testCollectionWithErrors(unittest.TestCase):
             self.collection.settings['name'],
             "Lesana collection with certain errors",
         )
-        self.assertEqual(len(self.collection.settings['fields']), 8)
+        self.assertEqual(len(self.collection.settings['fields']), 10)
         self.assertIsNotNone(self.collection.stemmer)
         self.assertEqual(len(self.collection.indexed_fields), 3)
 
@@ -508,6 +628,24 @@ class testCollectionCreation(unittest.TestCase):
         self.assertTrue(
             os.path.isfile(os.path.join(self.tmpdir, '.gitignore'))
         )
+        checkout_hook = os.path.join(
+            self.tmpdir,
+            '.git',
+            'hooks',
+            'post-checkout',
+        )
+        merge_hook = os.path.join(
+            self.tmpdir,
+            '.git',
+            'hooks',
+            'post-merge',
+        )
+        self.assertTrue(os.path.isfile(checkout_hook))
+        self.assertTrue(os.path.islink(merge_hook))
+        self.assertEqual(
+            os.path.abspath(checkout_hook),
+            os.path.abspath(os.readlink(merge_hook))
+        )
         # and then run it twice on the same directory, nothing should break
         collection = lesana.Collection.init(self.tmpdir)
         self.assertIsInstance(collection, lesana.Collection)
