@@ -4,6 +4,7 @@ import unittest
 
 import lesana
 from lesana import types
+from . import utils
 
 
 class DerivedType(types.LesanaString):
@@ -22,7 +23,7 @@ class Derivative(lesana.Collection):
 class testDerivatives(unittest.TestCase):
     def setUp(self):
         self.tmpdir = tempfile.mkdtemp()
-        shutil.copytree(
+        utils.copytree(
             'tests/data/derivative',
             self.tmpdir,
             dirs_exist_ok=True
